# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Helper for finding credentials.

When running this tool, there are two possible scenarios:

1. The tool is running as part of CI to publish the docs. In that case,
   credentials should be at a well-known location or use ADC.
2. The tool is running locally as part of development. In that case,
   the credentials should be passed into the command-line or via ADC.

(2) takes precedence. Command-line params also override local files.
"""

import os
from typing import List, Optional

from google.oauth2 import service_account
import google.auth

_WELL_KNOWN_LOCATIONS: List[str] = []

if "KOKORO_KEYSTORE_DIR" in os.environ:
    _WELL_KNOWN_LOCATIONS.append(
        os.path.join(
            os.environ["KOKORO_KEYSTORE_DIR"], "73713_docuploader_service_account"
        )
    )


def find_path():
    for location in _WELL_KNOWN_LOCATIONS:
        if os.path.exists(location):
            return location
    return ""


def find(credentials_file: Optional[str] = ""):
    if not credentials_file:
        credentials_file = find_path()

    if credentials_file != "":
        credentials = service_account.Credentials.from_service_account_file(
            credentials_file
        )
        return credentials, credentials.project_id
    else:
        return google.auth.default()



# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import os
import pathlib
import shutil
import sys
import tempfile
from typing import List, Optional

import click
from google.protobuf import text_format, json_format  # type: ignore
import pkg_resources

from docuploader.protos import metadata_pb2
import docuploader.credentials
import docuploader.log
import docuploader.tar
import docuploader.upload

try:
    VERSION = pkg_resources.get_distribution("gcp-docuploader").version
except pkg_resources.DistributionNotFound:
    VERSION = "0.0.0+dev"

DEFAULT_STAGING_BUCKET = "docs-resources"


@click.group()
@click.version_option(message="%(version)s", version=VERSION)
def main():
    pass


@main.command()
@click.option(
    "--staging-bucket",
    default=DEFAULT_STAGING_BUCKET,
    help="The bucket to upload the staged documentation to.",
)
@click.option(
    "--credentials",
    default=None,
    help="Path to the credentials file to use for Google Cloud Storage.",
)
@click.option(
    "--metadata-file",
    default=None,
    help="Path to the docs.metadata file. The path must be relative to the CWD.",
)
@click.option(
    "--destination-prefix",
    default=None,
    help="Prefix to include when uploading tar file. A - will be added after the prefix, if there is one.",
)
@click.argument("documentation_path")
def upload(
    staging_bucket: str,
    credentials: str,
    metadata_file: Optional[str],
    destination_prefix: str,
    documentation_path: str,
):
    credentials, project_id = docuploader.credentials.find(credentials)
    if not credentials:
        docuploader.log.error(
            "You need credentials to run this! Use Application Default Credentials or specify --credentials on the command line."
        )
        sys.exit(1)

    if metadata_file is None:
        metadata_file = "docs.metadata"
        json_file = pathlib.Path(documentation_path) / "docs.metadata.json"
        if json_file.exists():
            metadata_file = "docs.metadata.json"
    metadata_path = pathlib.Path(metadata_file)

    if not metadata_path.exists():
        docuploader.log.error(
            "You need metadata to upload the docs. You can generate it with docuploader create-metadata"
        )
        sys.exit(1)
    try:
        if not os.listdir(documentation_path):
            docuploader.log.error(
                f"The documentation path given ({documentation_path}) does not contain any documentation files."
            )
            sys.exit(1)
    except FileNotFoundError:
        docuploader.log.error(
            f"The documentation path given ({documentation_path}) does not exist relative to CWD."
        )
        sys.exit(1)
    except NotADirectoryError:
        docuploader.log.error(
            f"The documentation path given ({documentation_path}) is a file not a directory."
        )
        sys.exit(1)

    docuploader.log.success("Let's upload some docs!")

    docuploader.log.info("Loading up your metadata.")

    if documentation_path not in metadata_file:
        shutil.copy(
            metadata_path,
            pathlib.Path(documentation_path) / metadata_file,
        )

    metadata = metadata_pb2.Metadata()
    if metadata_file.endswith(".json"):
        json_format.Parse(metadata_path.read_text(), metadata)
    else:
        text_format.Merge(metadata_path.read_text(), metadata)

    # Validating metadata for required fields

    # TODO: Do additional validiation
    if not metadata.name:
        raise Exception("Metadata field 'name' is required.")
    if not metadata.version:
        raise Exception("Metadata field 'version' is required.")
    if not metadata.language:
        raise Exception("Metadata field 'language' is required.")

    docuploader.log.success(
        f"Looks like we're uploading {metadata.name} version {metadata.version} for {metadata.language}."
    )

    docuploader.log.info(
        f"Sit tight, I'm tarring up your docs in {documentation_path}."
    )

    with tempfile.NamedTemporaryFile() as fh:
        fh.close()
        tar_filename = fh.name
        docuploader.tar.compress(documentation_path, tar_filename)
        docuploader.log.success("Cool, I have those tarred up.")
        docuploader.log.info("Okay, I'm sending them to the cloud™ now.")
        destination_name = (
            f"{metadata.language}-{metadata.name}-{metadata.version}.tar.gz"
        )
        if destination_prefix:
            #  destination_prefix should end in alphanumeric character
            while not destination_prefix[-1].isalnum():
                destination_prefix = destination_prefix[:-1]

            destination_name = f"{destination_prefix}-{destination_name}"

        docuploader.upload.upload(
            source=tar_filename,
            destination=destination_name,
            bucket=staging_bucket,
            credentials=credentials,
            project_id=project_id,
        )

    docuploader.log.success(
        f"All is well, your docs were uploaded to gs://{staging_bucket}/{destination_name}! <3"
    )


@main.command()
# TODO: Use https://github.com/googleapis/protoc-docs-plugin to add docstrings
# to the pb2 module so that I can reference them here.
@click.option("--name", required=True)
@click.option("--version", required=True)
@click.option("--language", required=True)
@click.option("--distribution-name", default="")
@click.option("--product-page", default="")
@click.option("--github-repository", default="")
@click.option("--issue-tracker", default="")
@click.option("--stem", default="")
@click.option("--serving-path", default="")
@click.option("--xrefs", multiple=True, default=[])
@click.option("--xref-services", multiple=True, default=[])
@click.argument("destination", default="docs.metadata")
def create_metadata(
    name: str,
    version: str,
    language: str,
    distribution_name: str,
    product_page: str,
    github_repository: str,
    issue_tracker: str,
    destination: str,
    stem: str,
    serving_path: str,
    xrefs: List[str],
    xref_services: List[str],
):
    metadata = metadata_pb2.Metadata()
    metadata.update_time.FromDatetime(datetime.datetime.utcnow())
    metadata.name = name
    metadata.language = language
    metadata.version = version
    metadata.distribution_name = distribution_name
    metadata.product_page = product_page
    metadata.github_repository = github_repository
    metadata.issue_tracker = issue_tracker
    metadata.stem = stem
    metadata.serving_path = serving_path
    metadata.xrefs.extend(xrefs)
    metadata.xref_services.extend(xref_services)

    destination_path = pathlib.Path(destination)
    destination_path.write_text(text_format.MessageToString(metadata))
    docuploader.log.success(f"Wrote metadata to {destination_path}.")


if __name__ == "__main__":
    main()



# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Handles uploading files to Google Cloud Storage."""

from google.cloud import storage
from google.auth.credentials import Credentials


def upload(
    *,
    source: str,
    destination: str,
    bucket: str,
    credentials: Credentials,
    project_id: str
) -> storage.Blob:
    client = storage.Client(project=project_id, credentials=credentials)
    bucket_ = client.get_bucket(bucket)
    blob = bucket_.blob(destination)
    blob.upload_from_filename(filename=source)
    return blob



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: metadata.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0emetadata.proto\x12\x13yoshi.docs.metadata\x1a\x1fgoogle/protobuf/timestamp.proto\"\x99\x02\n\x08Metadata\x12/\n\x0bupdate_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07version\x18\x03 \x01(\t\x12\x10\n\x08language\x18\x04 \x01(\t\x12\x19\n\x11\x64istribution_name\x18\x05 \x01(\t\x12\x14\n\x0cproduct_page\x18\x06 \x01(\t\x12\x19\n\x11github_repository\x18\x07 \x01(\t\x12\x15\n\rissue_tracker\x18\x08 \x01(\t\x12\x0c\n\x04stem\x18\t \x01(\t\x12\x14\n\x0cserving_path\x18\n \x01(\t\x12\r\n\x05xrefs\x18\x0b \x03(\t\x12\x15\n\rxref_services\x18\x0c \x03(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'metadata_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _METADATA._serialized_start=73
  _METADATA._serialized_end=354
# @@protoc_insertion_point(module_scope)
